@@ -26,7 +26,7 @@ query = {
     'from': '1556625600000',  # unix timestamp in ms
     'to': '1556632800000',  # unix timestamp in ms
     'period': 1,
-    'filter': 'avg',
+    'filter': 'average',
     'dim.0': 'instance_id,6e83e6e7-3bf4-4b5b-b390-e80447ef5733',  # key, value
 }
 
