@@ -16,6 +16,7 @@ import openstack
 from openstack import _log
 from openstack import utils
 
+from otcextensions.sdk import proxy
 from otcextensions.sdk.compute.v2 import server
 from otcextensions.common import exc
 
@@ -54,6 +55,10 @@ OTC_SERVICES = {
         'endpoint_service_type': 'as',
         'append_project_id': True,
     },
+    'cbr': {
+        'service_type': 'cbr',
+        'append_project_id': True,
+    },
     'cce': {
         'service_type': 'cce',
         'endpoint_service_type': 'ccev2.0',
@@ -62,12 +67,6 @@ OTC_SERVICES = {
         'service_type': 'ces',
         'append_project_id': True,
     },
-    # 'ces': {
-    #    'service_type': 'ces',
-    #    'endpoint_service_type': 'cesv1',
-    #    'set_endpoint_override': True
-    #    'append_project_id': False,
-    # },
     'cts': {
         'service_type': 'cts',
     },
@@ -83,6 +82,7 @@ OTC_SERVICES = {
     },
     'dds': {
         'service_type': 'dds',
+        'endpoint_service_type': 'ddsv3',
     },
     'deh': {
         'service_type': 'deh',
@@ -146,6 +146,11 @@ OTC_SERVICES = {
     'sdrs': {
         'service_type': 'sdrs'
     },
+    'sfsturbo': {
+        'service_type': 'sfsturbo',
+        'endpoint_service_type': 'sfsturbo',
+        # 'append_project_id': True,
+    },
     'smn': {
         'service_type': 'smn',
         'append_project_id': True
@@ -160,7 +165,7 @@ OTC_SERVICES = {
     },
     'waf': {
         'service_type': 'waf',
-        'set_endpoint_override': True
+        # 'set_endpoint_override': True
     }
 }
 
@@ -266,6 +271,10 @@ def extend_instance(obj, cls):
 
 
 def patch_openstack_resources():
+    openstack.proxy.Proxy._report_stats_statsd = \
+        proxy.Proxy._report_stats_statsd
+    openstack.proxy.Proxy._report_stats_influxdb = \
+        proxy.Proxy._report_stats_influxdb
     openstack.compute.v2.server.Server._get_tag_struct = \
         server.Server._get_tag_struct
     openstack.compute.v2.server.Server.add_tag = server.Server.add_tag
