@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -15,6 +16,7 @@ from openstack.tests.unit import test_proxy_base
 from otcextensions.sdk.cce.v3 import _proxy
 from otcextensions.sdk.cce.v3 import cluster as _cluster
 from otcextensions.sdk.cce.v3 import cluster_node as _cluster_node
+from otcextensions.sdk.cce.v3 import node_pool as _node_pool
 from otcextensions.sdk.cce.v3 import cluster_cert as _cluster_cert
 from otcextensions.sdk.cce.v3 import job as _job
 
@@ -154,6 +156,73 @@ class TestCCEClusterNode(TestCCEProxy):
             expected_kwargs={'attribute': 'status.status'})
 
 
+class TestCCENodePool(TestCCEProxy):
+
+    def test_list_node_pool(self):
+        cluster = _cluster.Cluster(id='cluster_id')
+        self.verify_list(
+            self.proxy.node_pools, _node_pool.NodePool,
+            method_args=[cluster],
+            method_kwargs={},
+            expected_kwargs={
+                'paginated': False,
+                'cluster_id': cluster.id
+            }
+        )
+
+    def test_get_node_pool(self):
+        cluster = _cluster.Cluster(id='cluster_id')
+        self._verify2(
+            'openstack.proxy.Proxy._get',
+            self.proxy.get_node_pool,
+            method_args=[cluster, 'node_pool'],
+            expected_args=[_node_pool.NodePool, 'node_pool'],
+            expected_kwargs={
+                'cluster_id': cluster.id
+            }
+        )
+
+    def test_find_node_pool(self):
+        cluster = _cluster.Cluster(id='cluster_id')
+        self._verify2(
+            'openstack.proxy.Proxy._find',
+            self.proxy.find_node_pool,
+            method_args=[cluster, 'node_pool'],
+            expected_args=[_node_pool.NodePool, 'node_pool'],
+            expected_kwargs={
+                'cluster_id': cluster.id
+            }
+        )
+
+    def test_add_node_pool(self):
+        attrs = {'a': 'b'}
+        cluster = _cluster.Cluster(id='cluster_id')
+        self._verify2(
+            'openstack.proxy.Proxy._create',
+            self.proxy.create_node_pool,
+            method_args=[cluster],
+            method_kwargs=attrs,
+            expected_args=[_node_pool.NodePool],
+            expected_kwargs=dict(
+                cluster_id=cluster.id,
+                **attrs
+            )
+        )
+
+    def test_delete_node_pool(self):
+        cluster = _cluster.Cluster(id='cluster_id')
+        self._verify2(
+            'openstack.proxy.Proxy._delete',
+            self.proxy.delete_node_pool,
+            method_args=[cluster, 'n1'],
+            expected_args=[_node_pool.NodePool, 'n1'],
+            expected_kwargs={
+                'cluster_id': cluster.id,
+                'ignore_missing': True
+            }
+        )
+
+
 class TestCCEJob(TestCCEProxy):
     def test_get(self):
         self.verify_get(
